#!/usr/bin/perl -w

package ALTLinux::ACL;
use strict;
use warnings;
use File::Path qw(make_path);
use File::Basename;
our $VERSION = 0.02;
our $PKGNAME2ACL;
our $GRPNAME2ACL;
my $aclpackageprefix='list.packages.';
my $aclgroupprefix='list.groups.';
my @cachedirs=grep {$_} ($ENV{HOME}.'/.cache/altlinux-acl','.');

sub new {
    my $proto=shift;
    my $class=ref($proto) || $proto;
    my $self = {
	-expire=>5/24, # -M; 5/24=5 hours
	-flavour=> 'sisyphus',
	-verbose=> 1,
	@_,
    };
    bless($self,$class);
    unless ($PKGNAME2ACL) {
	$PKGNAME2ACL=$self->__load(-packages,$aclpackageprefix);
	$GRPNAME2ACL=$self->__load(-groups,$aclgroupprefix);
	# save from expansion
	$GRPNAME2ACL->{'@nobody'}=['@nobody'];
	$GRPNAME2ACL->{'@everybody'}=['@everybody'];
    }
    return $self;
}

sub acl {
    my ($self,$item)=@_;
    my $ptr=$PKGNAME2ACL->{$item};
    return unless $ptr;
    return @$ptr;
}

sub is_authorized {
    my ($self,$item, $user)=@_;
    my %expanded = map {$_=>1} $self->acl_expanded($item);
    return $expanded{$user} || $expanded{'@everybody'} || $expanded{'@nobody'};
}

sub is_leader {
    my ($self,$item, $user)=@_;
    my $raw_leader=$PKGNAME2ACL->{$item}->[0];
    return 1 if $raw_leader eq $user;
    return 0 if substr($raw_leader, 0, 1) ne '@';
    # indirect leader
    $raw_leader=$GRPNAME2ACL->{$raw_leader}->[0];
    return $raw_leader eq $user;
}

sub leader {
    my ($self,$item)=@_;
    my $raw_leader=$PKGNAME2ACL->{$item}->[0];
    $raw_leader=$GRPNAME2ACL->{$item}->[0] unless $raw_leader;
    return $raw_leader;
}

############ PUBLIC STATIC ###############
# i.e. called ALTLinux::ACL::<sub name>

sub email2aclname {
    my ($email)=@_;
    $email=~/^.+\<(\S+)\s*(?:\@|at)\s*altlinux.*>/;
    my $user=$1;
    unless ($user) {
	warn "e-mail [$email] does not look like ALT Linux maintainer e-mail.";
	return;
    }
    # for php-coder (acl name php_coder).
    $user=~s/-/_/g;
    return $user;
}

########## PROTECTED #################

# I'm not sure how to name it. If you use it, e-mail me.
sub acl_expanded {
    my ($self,$item)=@_;
    my $ptr=$PKGNAME2ACL->{$item};
    return unless $ptr;
    return map {$GRPNAME2ACL->{$_} ? @{$GRPNAME2ACL->{$_}} : $_ } @$ptr;
}

########## PRIVATE #################

sub __load {
    my ($self, $aclopt, $aclprefix)=@_;
    my $cache={};
    my @acllist = grep {defined($_) and -e $_} $self->{$aclopt}, map {$_.'/'.$aclprefix.$self->{-flavour}} @cachedirs;
    if (not @acllist) {
	$acllist[0]||=$self->{$aclopt};
	$acllist[0]||=$cachedirs[0].'/'.$aclprefix.$self->{-flavour};
    }
    # $acllist[0] contains creation timestamp, useful for wget speedup.
    # this is check timestamp.
    my $timestampfile=$acllist[0].'.timestamp';
    if (not -e $acllist[0] or not -e $timestampfile or -M $timestampfile > $self->{-expire}) {
	make_path(dirname($acllist[0]));
	my $url='http://git.altlinux.org/acl/'.$aclprefix.$self->{-flavour};
	my $retval=system ('wget','-q','-O'.$acllist[0],$url);
	print STDERR "updating acl cache: wget -q -O $acllist[0] $url\n" if $self->{-verbose};
	unless ($retval==0) {
	    unlink $acllist[0];
	    die "can't download acl file $url to $acllist[0] ($retval)" unless -e $acllist[0];
	}
	die "can't find acl file $acllist[0]" unless -e $acllist[0];
	system ('touch', $timestampfile);
    }

    open (ACL,'<',$acllist[0]) || die "can't open acl file $acllist[0]: $!";
    while (<ACL>) {
	chomp;
	if (/([^\t]+)\t(.+)/) {
	    my $name=$1;
	    my @acl=split (/\s+/,$2);
	    $cache->{$name}=\@acl if @acl;
	} else {
	    warn "acl: strange string $_";
	}
    }
    close(ACL);
    return $cache;
}

1;

=head1	NAME

ALTLinux::ACL - ALTLinux ACL query library.

=head1	DESCRIPTION

ALTLinux ACL query library automatically downloads, caches, and
provides an object interface to ALTLinux acl files.

=head1	AUTHOR

Written by Igor Vlasenko <viy@altlinux.org>.

=head1	COPYING

Copyright (c) 2010 Igor Vlasenko, ALT Linux Team.

This is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

=cut

